package eu.mhsl.minecraft.WorldMuseum.items.actionItems;

import eu.mhsl.minecraft.WorldMuseum.Main;
import eu.mhsl.minecraft.WorldMuseum.viewableWorld.ViewableWorld;
import net.kyori.adventure.text.Component;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.inventory.InventoryClickEvent;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.jetbrains.annotations.NotNull;

import java.util.List;

public class WorldSelector implements Listener, IActionItem {
    private final ItemStack ITEM = generateWorldSelectorItem();
    private final Inventory INVENTORY = generateWorldSelectorInventory();

    private ItemStack generateWorldSelectorItem() {
        ItemStack item = new ItemStack(Material.GRASS_BLOCK);
        ItemMeta meta = item.getItemMeta();
        meta.displayName(Component.text("Worldselector"));
        item.setItemMeta(meta);
        return item;
    }

    private Inventory generateWorldSelectorInventory() {
        List<ViewableWorld> worlds = Main.instance().getWorlds();

        int itemCount = worlds.size() + 1;
        int inventoryRows = Math.max(itemCount / 9, 1);
        int inventorySize = inventoryRows * 9;
        Inventory inventory = Bukkit.createInventory(null, inventorySize, Component.text("Weltenmuseum"));

        worlds.stream()
                .filter(ViewableWorld::isEnabled)
                .forEach(viewableWorld -> inventory.addItem(viewableWorld.getItem()));

        boolean canExit = Main.instance().getConfig().getBoolean("bungee.enabled");
        if(canExit) inventory.setItem(inventorySize - 1, exitItem());

        return inventory;
    }

    public static ItemStack exitItem() {
        ItemStack stack = ItemStack.of(Material.OAK_DOOR);
        ItemMeta meta = stack.getItemMeta();
        meta.displayName(Component.text("Zurück zum Server"));
        meta.setEnchantmentGlintOverride(true);
        stack.setItemMeta(meta);
        return stack;
    }

    @Override
    public void action(@NotNull Player player) {
        player.openInventory(INVENTORY);
    }

    @EventHandler
    public void onInteract(InventoryClickEvent event) {
        event.setCancelled(true);

        Main.instance().getWorlds().stream()
                .filter(viewableWorld -> viewableWorld.getItem().equals(event.getCurrentItem()))
                .findFirst()
                .ifPresent(viewableWorld -> viewableWorld.addViewer((Player) event.getWhoClicked()));
    }

    @Override
    public ItemStack getItem() {
        return ITEM;
    }

    @Override
    public int getItemLocation() {
        int itemLocation = 0; // Item will be placed into the first hotbar slot
        return itemLocation;
    }

}
